#include "stdafx.h"	// for MFC
#include "Network.h"

Network::Network()
{
	m_socket = NULL;
	m_acceptor = NULL;
}

Network::~Network()
{
	if (m_socket) {
		delete m_socket;
		//m_socket = NULL;
	}
	if (m_acceptor) {
		delete m_acceptor;
		//m_acceptor = NULL;
	}
}

void Network::set_socket(boost::asio::ip::tcp::socket *sock)
{
	m_socket = sock;
}

bool Network::connect( string address, int port )
{
	try	{
		tcp::resolver resolver(m_io_service);
		char ports[16];
		//itoa(port, ports, 10);
		_itoa_s(port, ports, sizeof(ports), 10);
		tcp::resolver::query query(tcp::v4(), address.c_str(), ports);
		tcp::resolver::iterator endpoint_iterator = resolver.resolve(query);
		tcp::resolver::iterator end;

		m_socket = new tcp::socket(m_io_service);
		boost::system::error_code error = boost::asio::error::host_not_found;
		m_socket->connect(*endpoint_iterator, error);
		if (error)
			throw boost::system::system_error(error);
	}
	catch (std::exception& e) {
		std::cerr << e.what() << std::endl;
		return false;
	}
	return true;
}


void Network::listen(int port, void (*server_connection)(Network&), bool background)
{
	m_io_service.reset();
	m_acceptor = new tcp::acceptor(m_io_service, tcp::endpoint(tcp::v4(), port));
//	cout << "listening on port " << port << "\n";
	if (background)
		boost::thread listenthread(boost::bind(boost::mem_fn(&Network::listen_helper), this, port, server_connection));
	else
		listen_helper(port, server_connection);
}

void Network::stop_listen()
{
	m_io_service.stop();
	m_acceptor->close();

	if (m_acceptor) {
		delete m_acceptor;
		m_acceptor = NULL;
	}
	if (m_socket) {
		delete m_socket;
		m_socket = NULL;
	}
}


void Network::listen_helper(int port, void (*server_connection)(Network&))
{
	TRACE("listen_helper() start\n");
	try	{
		for (;;) {
			m_socket = new tcp::socket(m_io_service);
			boost::system::error_code err;
			m_acceptor->accept(*m_socket,err);
			if (err) break;
			boost::thread serverconnectionthread(boost::bind(boost::mem_fn(&Network::server_connection_helper), this, m_socket, server_connection));
		}
	}
	catch (std::exception& e) {
		std::cerr << e.what() << std::endl;
	}
	TRACE("listen_helper() end\n");
}

void Network::server_connection_helper(tcp::socket *sock, void (*server_connection)(Network&))
{
	Network net;				// ꂪȂȂ͗ȂD
	net.set_socket(sock);	

//	cout << "new connection\n";
	TRACE("new connection\n");
	server_connection(net);			// NCAgɑ΂T[osD
//	cout << "connection closed\n";	
	TRACE("connection closed\n");	
}


// ̑M
bool Network::send_message(string message)
{
	boost::system::error_code errorcode;
	boost::asio::write(*m_socket, boost::asio::buffer(message), boost::asio::transfer_all(), errorcode);

	if (errorcode)
		return false;
	return true;
}

// oCif[^̑M
void Network::send_message(unsigned char *buffer, size_t n)
{
	if (n <= 0)
		return;

	boost::system::error_code e;
	size_t len = 0;

	do {
		len += m_socket->write_some(boost::asio::buffer(buffer, n-len), e);
	} while ( (len < n) && (!e) );

	if (len == n) {
		//cout << "successfully transferred " << len << " bytes\n";
	} else {
		cout << e.message() << "\n";
		throw std::exception(e.message().c_str());
	}
}



string Network::read_message()
{
	boost::system::error_code errorcode;
	boost::array<char, 2048> buf;

	size_t len = m_socket->read_some(boost::asio::buffer(buf), errorcode);

	if (errorcode) {
		cout << errorcode.message() << "\n";
		return string();
	}
	string msg(buf.data(), len);
	return msg;
}

unsigned char *Network::read_message(size_t n)
{
	if (n <= 0)
		return NULL;

	std::vector<unsigned char> packed_dimensions(n);
	boost::system::error_code e;
	size_t len = 0;

	do {
		len += m_socket->read_some(boost::asio::buffer(&packed_dimensions[len], n-len), e);
	} while ( (len < n) && (!e) );

	if (len == n) {
		//cout << "successfully transferred " << packed_dimensions.size() << " bytes\n";
	} else {
		cout << e.message() << "\n";
		throw std::exception(e.message().c_str());
	}

	unsigned char *data = new unsigned char[packed_dimensions.size()];

	for (unsigned int i = 0; i < packed_dimensions.size(); i++)
		data[i] = packed_dimensions[i];

	return data;
}